use std::{fs, path::PathBuf, sync::{atomic::{AtomicU64, Ordering}, Arc}};

use anyhow::Context;
use crossbeam_channel::Sender;
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize, Deserialize, Default)]
pub struct ModConfiguration {
    pub mod_id: String,
    pub mod_name: String,
    pub mod_description: String,
    pub mod_version: String,

    pub include_audio: bool,
    pub include_data: bool,
    pub include_fonts: bool,
    pub include_icon: bool,
    pub include_img: bool,
    pub include_plugins: bool,
    pub include_languages: bool,
    pub include_maps: bool,
    pub include_movies: bool
}
/*
    TODO:
    IMPORTANT!!!
        IMPLEMENT ERROR HANDLING AND PROPAGATION
        **BEFORE** MARKING 1.0.0!!!
    IMPORTANT!!
*/
fn walk_inner(relative_base: &PathBuf, real_filesystem_base: &PathBuf, target: &Sender<PathBuf>, counter: Arc<AtomicU64>) -> anyhow::Result<()> {
    let f = fs::read_dir(real_filesystem_base)?;
    for entry in f {
        let entry = entry?;
        let file_type = entry.file_type()?;
        let path = entry.path();
        let path = path.file_name().with_context(|| format!("The file doesn't have a name(????)"))?;

        let mut new_base = relative_base.clone();
        new_base.push(path);


        if file_type.is_dir() {
            let mut new_realfs_base = real_filesystem_base.clone();
            new_realfs_base.push(path);

            walk_inner(&new_base, &new_realfs_base, target, counter.clone())?;
        }
        if file_type.is_file() {
            counter.fetch_add(1, Ordering::Relaxed);
            target.send(new_base.clone())?;
        }
    }

    Ok(())
}

pub fn walk_dirs(base: &PathBuf, secondary_bases: &Vec<PathBuf>, target: Sender<PathBuf>, counter: Arc<AtomicU64>) {
    for path in secondary_bases {
        let mut real = base.clone();
        real.push(path);

        if walk_inner(path, &real, &target, counter.clone()).is_err() {
            todo!("Implement error handling you bitch");
        }
    }
}